@@ -28,6 +28,7 @@ from icepool.die.func import (standard, d, __getattr__, bernoulli, coin,
 
 from icepool.population import Population
 from icepool.die.die import Die
+from icepool.again import Again
 from icepool.die.die_with_truth import DieWithTruth
 
 from icepool.counts import CountsKeysView, CountsValuesView, CountsItemsView
@@ -36,7 +37,13 @@ from icepool.die.lowest_highest import lowest, highest
 
 from icepool.pool import Pool, standard_pool, clear_pool_cache
 from icepool.outcome_count_generator import OutcomeCountGenerator, NextOutcomeCountGenerator
-from icepool.outcome_count_evaluator import OutcomeCountEvaluator, WrapFuncEvaluator, JointEvaluator, EvaluateSum, evaluate_sum, enumerate_sorted, FindBestSet, FindBestRun
+from icepool.outcome_count_evaluator import OutcomeCountEvaluator, Order
+from icepool.evaluators import (
+    WrapFuncEvaluator, JointEvaluator, SumEvaluator, sum_evaluator,
+    expand_evaluator, CountInEvaluator, SubsetTargetEvaluator,
+    ContainsSubsetEvaluator, IntersectionSizeEvaluator,
+    BestMatchingSetEvaluator, best_matching_set_evaluator,
+    BestStraightEvaluator, best_straight_evaluator)
 
 from icepool.deck import Deck
 from icepool.deal import Deal
@@ -53,10 +60,11 @@ Reroll = SpecialValue.Reroll
 """Indicates an outcome should be rerolled (with no max depth)."""
 
 __all__ = [
-    'standard', 'd', 'bernoulli', 'coin', 'Die', 'Population', 'CountsKeysView',
-    'CountsValuesView', 'CountsItemsView', 'from_cumulative_quantities',
-    'from_rv', 'align', 'align_range', 'lowest', 'highest', 'min_outcome',
-    'max_outcome', 'reduce', 'accumulate', 'apply', 'apply_sorted', 'Reroll',
-    'OutcomeCountGenerator', 'Pool', 'standard_pool', 'OutcomeCountEvaluator',
-    'JointEvaluator', 'EvaluateSum', 'Deck', 'Deal', 'clear_pool_cache'
+    'standard', 'd', 'bernoulli', 'coin', 'Die', 'Population', 'Again',
+    'CountsKeysView', 'CountsValuesView', 'CountsItemsView',
+    'from_cumulative_quantities', 'from_rv', 'align', 'align_range', 'lowest',
+    'highest', 'min_outcome', 'max_outcome', 'reduce', 'accumulate', 'apply',
+    'apply_sorted', 'Reroll', 'OutcomeCountGenerator', 'Pool', 'standard_pool',
+    'OutcomeCountEvaluator', 'Order', 'JointEvaluator', 'SumEvaluator', 'Deck',
+    'Deal', 'clear_pool_cache'
 ]
