@@ -5,14 +5,22 @@ from icepool.alignment import Alignment
 
 from abc import ABC, abstractmethod
 from collections import defaultdict
+import enum
 from functools import cached_property
 import itertools
 import math
 
 from typing import Any, Callable, Collection, Hashable, Mapping, MutableMapping, Sequence
 
-PREFERRED_DIRECTION_COST_FACTOR = 10
-"""The preferred direction will be favored this times as much."""
+PREFERRED_ORDER_COST_FACTOR = 10
+"""The preferred order will be favored this times as much."""
+
+
+class Order(enum.IntEnum):
+    """Can be used to define what order outcomes are seen in by OutcomeCountEvaluators."""
+    Ascending = 1
+    Descending = -1
+    Any = 0
 
 
 class OutcomeCountEvaluator(ABC):
@@ -72,7 +80,7 @@ class OutcomeCountEvaluator(ABC):
                 `state` will be `None`.
             outcome: The current outcome.
                 `next_state` will see all rolled outcomes in monotonic order;
-                either ascending or descending depending on `direction()`.
+                either ascending or descending depending on `order()`.
                 If there are multiple generators, the set of outcomes is the
                 union of the outcomes of the invididual generators. All outcomes
                 with nonzero count will be seen. Outcomes with zero count
@@ -118,8 +126,8 @@ class OutcomeCountEvaluator(ABC):
         """
         return final_state
 
-    def direction(self, *generators: icepool.OutcomeCountGenerator) -> int:
-        """Optional function to determine the direction in which `next_state()` will see outcomes.
+    def order(self, *generators: icepool.OutcomeCountGenerator) -> Order:
+        """Optional function to determine the order in which `next_state()` will see outcomes.
 
         There is no expectation that a subclass be able to handle
         an arbitrary number of generators. Thus, you are free to rename any of
@@ -136,15 +144,18 @@ class OutcomeCountEvaluator(ABC):
                 parameters.
 
         Returns:
-            * > 0 if `next_state()` should always see the outcomes in ascending order.
-            * < 0 if `next_state()` should always see the outcomes in descending order.
-            * 0 if the order may be determined automatically.
+            * Order.Ascending (= 1)
+                if `next_state()` should always see the outcomes in ascending order.
+            * Order.Descending (= -1)
+                if `next_state()` should always see the outcomes in descending order.
+            * Order.Any (= 0)
+                if the result of the evaluation is order-independent.
         """
-        return 1
+        return Order.Ascending
 
     def alignment(self,
                   *generators: icepool.OutcomeCountGenerator) -> Collection:
-        """Optional function to specify an collection of outcomes that should always be given to `next_state()` even if they have zero count.
+        """Optional method to specify an collection of outcomes that should always be given to `next_state()` even if they have zero count.
 
         There is no expectation that a subclass be able to handle
         an arbitrary number of generators. Thus, you are free to rename any of
@@ -194,9 +205,15 @@ class OutcomeCountEvaluator(ABC):
         max_outcome = max(generator.max_outcome() for generator in generators)
         return range(min_outcome, max_outcome + 1)
 
+    def final_kwargs(
+            self,
+            *generators: icepool.OutcomeCountGenerator) -> Mapping[str, Any]:
+        """Optional method to specify any extra keyword arguments to the final die constructor."""
+        return {}
+
     @cached_property
     def _cache(self) -> MutableMapping[Any, Mapping[Any, int]]:
-        """A cache of (direction, generators) -> weight distribution over states. """
+        """A cache of (order, generators) -> weight distribution over states. """
         return {}
 
     def evaluate(
@@ -206,7 +223,7 @@ class OutcomeCountEvaluator(ABC):
         """Evaluates generator(s).
 
         You can call the `OutcomeCountEvaluator` object directly for the same effect,
-        e.g. `evaluate_sum(generator)` is an alias for `evaluate_sum.evaluate(generator)`.
+        e.g. `sum_evaluator(generator)` is an alias for `sum_evaluator.evaluate(generator)`.
 
         Most evaluators will expect a fixed number of generators.
         The union of the outcomes of the generator(s) must be totally orderable.
@@ -231,12 +248,12 @@ class OutcomeCountEvaluator(ABC):
                    for generator in converted_generators):
             return icepool.Die([])
 
-        algorithm, direction = self._select_algorithm(*converted_generators)
+        algorithm, order = self._select_algorithm(*converted_generators)
 
         # We use a separate class to guarantee all outcomes are visited.
         alignment = Alignment(self.alignment(*converted_generators))
 
-        dist = algorithm(direction, alignment, tuple(converted_generators))
+        dist = algorithm(order, alignment, tuple(converted_generators))
 
         final_outcomes = []
         final_weights = []
@@ -252,64 +269,65 @@ class OutcomeCountEvaluator(ABC):
                 final_outcomes.append(outcome)
                 final_weights.append(weight)
 
-        return icepool.Die(final_outcomes, final_weights)
+        return icepool.Die(final_outcomes, final_weights,
+                           **self.final_kwargs(*converted_generators))
 
     __call__ = evaluate
 
     def _select_algorithm(
-            self,
-            *generators: icepool.OutcomeCountGenerator) -> tuple[Callable, int]:
-        """Selects an algorithm and iteration direction.
+            self, *generators: icepool.OutcomeCountGenerator
+    ) -> tuple[Callable, Order]:
+        """Selects an algorithm and iteration order.
 
         Returns:
             * The algorithm to use (`_eval_internal*`).
-            * The direction in which `next_state()` sees outcomes.
+            * The order in which `next_state()` sees outcomes.
                 1 for ascending and -1 for descending.
         """
-        eval_direction = self.direction(*generators)
+        eval_order = self.order(*generators)
 
-        pop_min_costs, pop_max_costs = zip(*(
-            generator._estimate_direction_costs() for generator in generators))
+        pop_min_costs, pop_max_costs = zip(
+            *(generator._estimate_order_costs() for generator in generators))
 
         pop_min_cost = math.prod(pop_min_costs)
         pop_max_cost = math.prod(pop_max_costs)
 
-        # No preferred direction case: go directly with cost.
-        if eval_direction == 0:
+        # No preferred order case: go directly with cost.
+        if eval_order == Order.Any:
             if pop_max_cost <= pop_min_cost:
-                return self._eval_internal, 1
+                return self._eval_internal, Order.Ascending
             else:
-                return self._eval_internal, -1
+                return self._eval_internal, Order.Descending
 
-        # Preferred direction case.
-        # Go with the preferred direction unless there is a "significant"
+        # Preferred order case.
+        # Go with the preferred order unless there is a "significant"
         # cost factor.
 
-        if PREFERRED_DIRECTION_COST_FACTOR * pop_max_cost < pop_min_cost:
-            cost_direction = 1
-        elif PREFERRED_DIRECTION_COST_FACTOR * pop_min_cost < pop_max_cost:
-            cost_direction = -1
+        if PREFERRED_ORDER_COST_FACTOR * pop_max_cost < pop_min_cost:
+            cost_order = Order.Ascending
+        elif PREFERRED_ORDER_COST_FACTOR * pop_min_cost < pop_max_cost:
+            cost_order = Order.Descending
         else:
-            cost_direction = 0
+            cost_order = Order.Any
 
-        if cost_direction == 0 or eval_direction == cost_direction:
+        if cost_order == Order.Any or eval_order == cost_order:
             # Use the preferred algorithm.
-            return self._eval_internal, eval_direction
+            return self._eval_internal, eval_order
         else:
             # Use the less-preferred algorithm.
-            return self._eval_internal_iterative, eval_direction
+            return self._eval_internal_iterative, eval_order
 
     def _eval_internal(
-        self, direction: int, alignment: Alignment,
+        self, order: int, alignment: Alignment,
         generators: tuple[icepool.OutcomeCountGenerator,
                           ...]) -> Mapping[Any, int]:
-        """Internal algorithm for iterating in the more-preferred direction,
+        """Internal algorithm for iterating in the more-preferred order,
         i.e. giving outcomes to `next_state()` from wide to narrow.
 
         All intermediate return values are cached in the instance.
 
         Arguments:
-            direction: The direction in which to send outcomes to `next_state()`.
+            order: The order in which to send outcomes to `next_state()`.
             alignment: As `alignment()`. Elements will be popped off this
                 during recursion.
             generators: One or more `OutcomeCountGenerators`s to evaluate. Elements
@@ -319,7 +337,7 @@ class OutcomeCountEvaluator(ABC):
             A dict `{ state : weight }` describing the probability distribution
                 over states.
         """
-        cache_key = (direction, alignment, generators)
+        cache_key = (order, alignment, generators)
         if cache_key in self._cache:
             return self._cache[cache_key]
 
@@ -330,12 +348,12 @@ class OutcomeCountEvaluator(ABC):
             result = {None: 1}
         else:
             outcome, prev_alignment, iterators = OutcomeCountEvaluator._pop_generators(
-                direction, alignment, generators)
+                order, alignment, generators)
             for p in itertools.product(*iterators):
                 prev_generators, counts, weights = zip(*p)
                 counts = tuple(itertools.chain.from_iterable(counts))
                 prod_weight = math.prod(weights)
-                prev = self._eval_internal(direction, prev_alignment,
+                prev = self._eval_internal(order, prev_alignment,
                                            prev_generators)
                 for prev_state, prev_weight in prev.items():
                     state = self.next_state(prev_state, outcome, *counts)
@@ -346,10 +364,10 @@ class OutcomeCountEvaluator(ABC):
         return result
 
     def _eval_internal_iterative(
-        self, direction: int, alignment: Alignment,
+        self, order: int, alignment: Alignment,
         generators: tuple[icepool.OutcomeCountGenerator,
                           ...]) -> Mapping[Any, int]:
-        """Internal algorithm for iterating in the less-preferred direction,
+        """Internal algorithm for iterating in the less-preferred order,
         i.e. giving outcomes to `next_state()` from narrow to wide.
 
         This algorithm does not perform persistent memoization.
@@ -364,9 +382,9 @@ class OutcomeCountEvaluator(ABC):
             next_dist: MutableMapping[Any, int] = defaultdict(int)
             for (prev_state, prev_alignment,
                  prev_generators), weight in dist.items():
-                # The direction flip here is the only purpose of this algorithm.
+                # The order flip here is the only purpose of this algorithm.
                 outcome, alignment, iterators = OutcomeCountEvaluator._pop_generators(
-                    -direction, prev_alignment, prev_generators)
+                    -order, prev_alignment, prev_generators)
                 for p in itertools.product(*iterators):
                     generators, counts, weights = zip(*p)
                     counts = tuple(itertools.chain.from_iterable(counts))
@@ -430,187 +448,3 @@ class OutcomeCountEvaluator(ABC):
             return result.outcomes()[0]
         else:
             return result
-
-
-class WrapFuncEvaluator(OutcomeCountEvaluator):
-    """An `OutcomeCountEvaluator` created from a single provided function.
-
-    `next_state()` simply calls that function.
-    """
-
-    def __init__(self, func: Callable, /):
-        """Constructs a new instance given the function that should be called for `next_state()`.
-        Args:
-            func(state, outcome, *counts): This should take the same arguments
-                as `next_state()`, minus `self`, and return the next state.
-        """
-        self._func = func
-
-    def next_state(self, state: Hashable, outcome, *counts: int) -> Hashable:
-        return self._func(state, outcome, *counts)
-
-
-class JointEvaluator(OutcomeCountEvaluator):
-    """EXPERIMENTAL: An `OutcomeCountEvaluator` that jointly evaluates sub-evaluators on the same roll(s) of a generator.
-
-    It may be more efficient to write the joint evaluation directly; this is
-    provided as a convenience.
-    """
-
-    def __init__(self, *sub_evaluators: 'OutcomeCountEvaluator'):
-        self._sub_evaluators = sub_evaluators
-
-    def next_state(self, state, outcome, *counts: int) -> tuple[Hashable, ...]:
-        """Runs `next_state` for all subevals.
-
-        The state is a tuple of the substates.
-        """
-        if state is None:
-            return tuple(
-                evaluator.next_state(None, outcome, *counts)
-                for evaluator in self._sub_evaluators)
-        else:
-            return tuple(
-                evaluator.next_state(substate, outcome, *counts)
-                for evaluator, substate in zip(self._sub_evaluators, state))
-
-    def final_outcome(self, final_state,
-                      *generators: icepool.OutcomeCountGenerator):
-        """Runs `final_state` for all subevals.
-
-        The final outcome is a tuple of the final suboutcomes.
-        """
-        return tuple(
-            evaluator.final_outcome(final_substate, *generators) for evaluator,
-            final_substate in zip(self._sub_evaluators, final_state))
-
-    def direction(self, *generators: icepool.OutcomeCountGenerator) -> int:
-        """Determines the common direction of the subevals.
-
-        Raises:
-            ValueError: If subevals have conflicting directions, i.e. some are
-                ascending and others are descending.
-        """
-        subdirections = tuple(
-            evaluator.direction(*generators)
-            for evaluator in self._sub_evaluators)
-        ascending = any(x > 0 for x in subdirections)
-        descending = any(x < 0 for x in subdirections)
-        if ascending and descending:
-            raise ValueError('Sub-evals have conflicting directions.')
-        elif ascending:
-            return 1
-        elif descending:
-            return -1
-        else:
-            return 0
-
-
-class EvaluateSum(OutcomeCountEvaluator):
-    """A simple `OutcomeCountEvaluator` that just sums the outcomes in a generator. """
-
-    def next_state(self, state, outcome, count):
-        """Add the outcomes to the running total. """
-        if state is None:
-            return outcome * count
-        else:
-            return state + outcome * count
-
-    def final_outcome(self, final_state, *_):
-        if final_state is None:
-            return 0
-        else:
-            return final_state
-
-    def direction(self, *_):
-        """This evaluator doesn't care about direction. """
-        return 0
-
-
-evaluate_sum = EvaluateSum()
-"""A shared `Sumgenerators` object for caching results. """
-
-
-class EnumerateSorted(OutcomeCountEvaluator):
-    """A `OutcomeCountEvaluator` that enumerates all (sorted) results.
-
-    This is expensive and not recommended unless there are few elements being output.
-    """
-
-    def __init__(self, direction=1):
-        """`direction` determines the sort order.
-
-        Positive for ascending and negative for descending.
-        """
-        self._direction = direction
-
-    def next_state(self, state, outcome, count):
-        if count < 0:
-            raise ValueError(
-                'EnumerateGenerator is not compatible with negative counts.')
-        if state is None:
-            return (outcome,) * count
-        else:
-            return state + (outcome,) * count
-
-    def direction(self, *_):
-        return self._direction
-
-
-enumerate_sorted = EnumerateSorted()
-"""A shared `EnumerateGenerator` object for caching results. """
-
-
-class FindBestSet(OutcomeCountEvaluator):
-    """A `OutcomeCountEvaluator` that takes the best matching set in a generator.
-
-    This prioritizes set size, then the outcome.
-
-    The outcomes are `(set_size, outcome)`.
-    """
-
-    def next_state(self, state, outcome, count):
-        """Replace the last best set if this one is better.
-
-        Note the use of tuple comparison, which priortizes elements to the left.
-        """
-        if state is None:
-            return count, outcome
-        else:
-            return max(state, (count, outcome))
-
-    def direction(self, *_):
-        """This evaluator doesn't care about direction. """
-        return 0
-
-
-class FindBestRun(OutcomeCountEvaluator):
-    """A `OutcomeCountEvaluator` that takes the best run (aka "straight") in a generator.
-
-    Outcomes must be `int`s.
-
-    This prioritizes run size, then the outcome.
-
-    The outcomes are `(run_size, outcome)`.
-    """
-
-    def next_state(self, state, outcome, count):
-        """Increments the current run if at least one `Die` rolled this outcome,
-        then saves the run to the state.
-        """
-        best_run, best_run_outcome, run = state or (0, outcome, 0)
-        if count >= 1:
-            run += 1
-        else:
-            run = 0
-        return max((run, outcome), (best_run, best_run_outcome)) + (run,)
-
-    def final_outcome(self, final_state, *_):
-        """The best run. """
-        return final_state[:2]
-
-    def direction(self, *_):
-        """This only considers outcomes in ascending order. """
-        return 1
-
-    alignment = OutcomeCountEvaluator.range_alignment
